"use strict";

// https://github.com/WebReflection/url-search-params
var Symbol = require('./symbol');

var find = /[!'\(\)~]|%20|%00/g;
var plus = /\+/g;
var replace = {
  '!': '%21',
  "'": '%27',
  '(': '%28',
  ')': '%29',
  '~': '%7E',
  '%20': '+',
  '%00': '\x00'
};

var replacer = function replacer(match) {
  return replace[match];
};

var secret = '__URLSearchParams__';

function encode(str) {
  return encodeURIComponent(str).replace(find, replacer);
}

function decode(str) {
  return decodeURIComponent(str.replace(plus, ' '));
}

var URLSearchParams =
/*#__PURE__*/
function () {
  function URLSearchParams(query) {
    this[secret] = Object.create(null);
    if (!query) return;

    if (query.charAt(0) === '?') {
      query = query.slice(1);
    }

    for (var index, value, pairs = (query || '').split('&'), i = 0, length = pairs.length; i < length; i++) {
      value = pairs[i];
      index = value.indexOf('=');

      if (-1 < index) {
        this.append(decode(value.slice(0, index)), decode(value.slice(index + 1)));
      } else if (value.length) {
        this.append(decode(value), '');
      }
    }
  }

  var _proto = URLSearchParams.prototype;

  _proto.append = function append(name, value) {
    var dict = this[secret];

    if (name in dict) {
      dict[name].push('' + value);
    } else {
      dict[name] = ['' + value];
    }
  };

  _proto.delete = function _delete(name) {
    delete this[secret][name];
  };

  _proto.get = function get(name) {
    var dict = this[secret];
    return name in dict ? dict[name][0] : null;
  };

  _proto.getAll = function getAll(name) {
    var dict = this[secret];
    return name in dict ? dict[name].slice(0) : [];
  };

  _proto.has = function has(name) {
    return name in this[secret];
  };

  _proto.set = function set(name, value) {
    this[secret][name] = ['' + value];
  };

  _proto.forEach = function forEach(callback, thisArg) {
    var dict = this[secret];
    Object.getOwnPropertyNames(dict).forEach(function (name) {
      dict[name].forEach(function (value) {
        callback.call(thisArg, value, name, this);
      }, this);
    }, this);
  };

  _proto.keys = function keys() {
    var items = [];
    this.forEach(function (value, name) {
      items.push(name);
    });
    var iterator = {
      next: function next() {
        var value = items.shift();
        return {
          done: value === undefined,
          value: value
        };
      }
    };

    iterator[Symbol.iterator] = function () {
      return iterator;
    };

    return iterator;
  };

  _proto.values = function values() {
    var items = [];
    this.forEach(function (value) {
      items.push(value);
    });
    var iterator = {
      next: function next() {
        var value = items.shift();
        return {
          done: value === undefined,
          value: value
        };
      }
    };

    iterator[Symbol.iterator] = function () {
      return iterator;
    };

    return iterator;
  };

  _proto.entries = function entries() {
    var items = [];
    this.forEach(function (value, name) {
      items.push([name, value]);
    });
    var iterator = {
      next: function next() {
        var value = items.shift();
        return {
          done: value === undefined,
          value: value
        };
      }
    };

    iterator[Symbol.iterator] = function () {
      return iterator;
    };

    return iterator;
  };

  _proto.toString = function toString() {
    var dict = this[secret],
        query = [],
        i,
        key,
        name,
        value;

    for (key in dict) {
      name = encode(key);

      for (i = 0, value = dict[key]; i < value.length; i++) {
        query.push(name + '=' + encode(value[i]));
      }
    }

    return query.join('&');
  };

  return URLSearchParams;
}();

URLSearchParams.prototype[Symbol.iterator] = URLSearchParams.prototype.entries;
module.exports = URLSearchParams;